==> java-analysis-impl-222.3345.118-sources.jar: com/siyeh/ig/dataflow/LawOfDemeterInspection.java <==
// Copyright 2000-2022 JetBrains s.r.o. and contributors. Use of this source code is governed by the Apache 2.0 license.
package com.siyeh.ig.dataflow;

import com.intellij.codeInspection.ui.SingleCheckboxOptionsPanel;
import com.intellij.psi.*;
import com.intellij.psi.util.PsiUtil;
import com.siyeh.InspectionGadgetsBundle;
import com.siyeh.ig.BaseInspection;
import com.siyeh.ig.BaseInspectionVisitor;
import com.siyeh.ig.psiutils.DeclarationSearchUtils;
import com.siyeh.ig.psiutils.MethodUtils;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import javax.swing.*;

public class LawOfDemeterInspection extends BaseInspection {

  @SuppressWarnings({"PublicField"})
  public boolean ignoreLibraryCalls = true;

  @Override
  @NotNull
  protected String buildErrorString(Object... infos) {
    boolean isMethodCall = (Boolean)infos[0];
    return isMethodCall
           ? InspectionGadgetsBundle.message("law.of.demeter.problem.descriptor")
           : InspectionGadgetsBundle.message("law.of.demeter.field.problem.descriptor");
  }

  @Override
  @Nullable
  public JComponent createOptionsPanel() {
    return new SingleCheckboxOptionsPanel(InspectionGadgetsBundle.message("law.of.demeter.ignore.library.calls.option"), this,
                                          "ignoreLibraryCalls");
  }

  @Override
  public BaseInspectionVisitor buildVisitor() {
    return new LawOfDemeterVisitor();
  }

  private class LawOfDemeterVisitor extends BaseInspectionVisitor {

    @Override
    public void visitMethodCallExpression(PsiMethodCallExpression expression) {
      super.visitMethodCallExpression(expression);
      final PsiExpression qualifier = expression.getMethodExpression().getQualifierExpression();
      if (qualifier == null) {
        return;
      }
      if (!isCallToSuspiciousMethod(expression)) {
        return;
      }
      if (violatesLawOfDemeter(qualifier)) {
        registerMethodCallError(expression, Boolean.TRUE);
      }
    }

    @Override
    public void visitReferenceExpression(PsiReferenceExpression expression) {
      super.visitReferenceExpression(expression);
      if (expression.getParent() instanceof PsiMethodCallExpression) {
        return;
      }
      final PsiExpression qualifier = expression.getQualifierExpression();
      if (qualifier == null) {
        return;
      }
      final PsiElement nameElement = expression.getReferenceNameElement();
      if (nameElement == null) {
        return;
      }
      if (violatesLawOfDemeter(qualifier)) {
        registerError(nameElement, Boolean.FALSE);
      }
    }

    private boolean isCallToSuspiciousMethod(PsiMethodCallExpression expression) {
      final PsiMethod method = expression.resolveMethod();
      if (method == null || MethodUtils.isFactoryMethod(method)) {
        return false;
      }
      if (ignoreLibraryCalls && method instanceof PsiCompiledElement) {
        return false;
      }
      if (method.getContainingFile() == expression.getContainingFile()) {
        // consider code in the same file a "friend"
        return false;
      }
      final PsiType type = method.getReturnType();
      final PsiClass aClass = PsiUtil.resolveClassInClassTypeOnly(type);
      if (method.getContainingClass() == aClass) {
        return false;
      }
      return true;
    }

    public boolean violatesLawOfDemeter(PsiExpression expression) {
      final PsiElement qualifier = PsiUtil.skipParenthesizedExprUp(expression);
      if (qualifier instanceof PsiReferenceExpression) {
        final PsiReferenceExpression referenceExpression = (PsiReferenceExpression)qualifier;
        final PsiElement target = referenceExpression.resolve();
        if (target instanceof PsiParameter) {
          return false;
        }
        else if (target instanceof PsiField) {
          final PsiField field = (PsiField)target;
          if (field.hasModifierProperty(PsiModifier.STATIC)) {
            return false;
          }
          else if (field.getContainingFile() == expression.getContainingFile()) {
            return false;
          }
          else if (ignoreLibraryCalls && field instanceof PsiCompiledElement) {
            return false;
          }
          return true;
        }
        else if (target instanceof PsiLocalVariable) {
          final PsiExpression definition = DeclarationSearchUtils.findDefinition(referenceExpression, (PsiVariable)target);
          return violatesLawOfDemeter(definition);
        }
      }
      else if (qualifier instanceof PsiMethodCallExpression) {
        final PsiMethodCallExpression methodCallExpression = (PsiMethodCallExpression)qualifier;
        if (isCallToSuspiciousMethod(methodCallExpression)) {
          return true;
        }
      }
      return false;
    }
  }
}

==> java-analysis-impl-223.7571.182-sources.jar: com/siyeh/ig/dataflow/LawOfDemeterInspection.java <==
// Copyright 2000-2022 JetBrains s.r.o. and contributors. Use of this source code is governed by the Apache 2.0 license.
package com.siyeh.ig.dataflow;

import com.intellij.codeInspection.ui.SingleCheckboxOptionsPanel;
import com.intellij.psi.*;
import com.intellij.psi.util.PsiUtil;
import com.siyeh.InspectionGadgetsBundle;
import com.siyeh.ig.BaseInspection;
import com.siyeh.ig.BaseInspectionVisitor;
import com.siyeh.ig.psiutils.DeclarationSearchUtils;
import com.siyeh.ig.psiutils.MethodUtils;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import javax.swing.*;

public class LawOfDemeterInspection extends BaseInspection {

  @SuppressWarnings({"PublicField"})
  public boolean ignoreLibraryCalls = true;

  @Override
  @NotNull
  protected String buildErrorString(Object... infos) {
    boolean isMethodCall = (Boolean)infos[0];
    return isMethodCall
           ? InspectionGadgetsBundle.message("law.of.demeter.problem.descriptor")
           : InspectionGadgetsBundle.message("law.of.demeter.field.problem.descriptor");
  }

  @Override
  @Nullable
  public JComponent createOptionsPanel() {
    return new SingleCheckboxOptionsPanel(InspectionGadgetsBundle.message("law.of.demeter.ignore.library.calls.option"), this,
                                          "ignoreLibraryCalls");
  }

  @Override
  public BaseInspectionVisitor buildVisitor() {
    return new LawOfDemeterVisitor();
  }

  private class LawOfDemeterVisitor extends BaseInspectionVisitor {

    @Override
    public void visitMethodCallExpression(@NotNull PsiMethodCallExpression expression) {
      super.visitMethodCallExpression(expression);
      final PsiExpression qualifier = expression.getMethodExpression().getQualifierExpression();
      if (qualifier == null) {
        return;
      }
      if (!isCallToSuspiciousMethod(expression)) {
        return;
      }
      if (violatesLawOfDemeter(qualifier)) {
        registerMethodCallError(expression, Boolean.TRUE);
      }
    }

    @Override
    public void visitReferenceExpression(@NotNull PsiReferenceExpression expression) {
      super.visitReferenceExpression(expression);
      if (expression.getParent() instanceof PsiMethodCallExpression) {
        return;
      }
      final PsiExpression qualifier = expression.getQualifierExpression();
      if (qualifier == null) {
        return;
      }
      final PsiElement nameElement = expression.getReferenceNameElement();
      if (nameElement == null) {
        return;
      }
      if (violatesLawOfDemeter(qualifier)) {
        registerError(nameElement, Boolean.FALSE);
      }
    }

    private boolean isCallToSuspiciousMethod(PsiMethodCallExpression expression) {
      final PsiMethod method = expression.resolveMethod();
      if (method == null || MethodUtils.isFactoryMethod(method)) {
        return false;
      }
      if (ignoreLibraryCalls && method instanceof PsiCompiledElement) {
        return false;
      }
      if (method.getContainingFile() == expression.getContainingFile()) {
        // consider code in the same file a "friend"
        return false;
      }
      final PsiType type = method.getReturnType();
      final PsiClass aClass = PsiUtil.resolveClassInClassTypeOnly(type);
      if (method.getContainingClass() == aClass) {
        return false;
      }
      return true;
    }

    public boolean violatesLawOfDemeter(PsiExpression expression) {
      final PsiElement qualifier = PsiUtil.skipParenthesizedExprUp(expression);
      if (qualifier instanceof PsiReferenceExpression) {
        final PsiReferenceExpression referenceExpression = (PsiReferenceExpression)qualifier;
        final PsiElement target = referenceExpression.resolve();
        if (target instanceof PsiParameter) {
          return false;
        }
        else if (target instanceof PsiField) {
          final PsiField field = (PsiField)target;
          if (field.hasModifierProperty(PsiModifier.STATIC)) {
            return false;
          }
          else if (field.getContainingFile() == expression.getContainingFile()) {
            return false;
          }
          else if (ignoreLibraryCalls && field instanceof PsiCompiledElement) {
            return false;
          }
          return true;
        }
        else if (target instanceof PsiLocalVariable) {
          final PsiExpression definition = DeclarationSearchUtils.findDefinition(referenceExpression, (PsiVariable)target);
          return violatesLawOfDemeter(definition);
        }
      }
      else if (qualifier instanceof PsiMethodCallExpression) {
        final PsiMethodCallExpression methodCallExpression = (PsiMethodCallExpression)qualifier;
        if (isCallToSuspiciousMethod(methodCallExpression)) {
          return true;
        }
      }
      return false;
    }
  }
}
